
function [im_reco, e, V, Media, D, energia]=reconstruccion(numero)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%    function [im_reco, e, V, Media, D, energia]=reconstruccion(numero)
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Esta funcion calcula el eigenespacio asociado a 9 imagenes de 9 pilotos de motos
% calcula la reconstruccin de uno de ellos variando el numero de vectores en la reconstruccion
%
%
% numero es el piloto a proyectar y reconstruir
%
% im_reco es la  matriz con las imagenes reconstruidas
% e es el error de reconstruccion de cada imagen reconstruida
% V es la matriz de los eigenvectores
% Media es la imagen media de los pilotos
% D son los eigenvalores
% E es la energia


display=1; %Variable que controla la visualizacin (1) o no (0) de los datos y su reconstruccion
%each column is an example 
N=9; %number of examples (numero de pilotos)
Ny_original=254; %tamanyo de la imagen
Nx_original=273;
Ny=round(Ny_original/4);
Nx=round(Nx_original/4);
M=Nx*Ny; %dimension of each image
Nv=N-1; %numero de vectores

x1=imread('pilotos/rossi.jpg');
x2=imread('pilotos/sete.jpg');
x3=imread('pilotos/pedrosa.jpg');
x4=imread('pilotos/jorgelorenzo.jpg');
x5=imread('pilotos/capirossi.jpg');
x6=imread('pilotos/biaggi.jpg');
x7=imread('pilotos/elias.jpg');
x8=imread('pilotos/rolfo.jpg');
x9=imread('pilotos/poggiali.jpg');

x1=imresize(x1,[Nx Ny],'bilinear');
x2=imresize(x2,[Nx Ny],'bilinear');
x3=imresize(x3,[Nx Ny],'bilinear');
x4=imresize(x4,[Nx Ny],'bilinear');
x5=imresize(x5,[Nx Ny],'bilinear');
x6=imresize(x6,[Nx Ny],'bilinear');
x7=imresize(x7,[Nx Ny],'bilinear');
x8=imresize(x8,[Nx Ny],'bilinear');
x9=imresize(x9,[Nx Ny],'bilinear');

%Las imagenes de los pilotos son en color, luego cada matriz es (273 254 3)
%las abro y concateno en un solo vector columna

for i=1:N
comando=sprintf('x%d=[x%d(:,:,1) x%d(:,:,2) x%d(:,:,3)];',i,i,i,i);
eval(comando);
end


for i=1:N
comando=sprintf('x%d=x%d(:);',i,i);
eval(comando);
end


X=[x1 x2 x3 x4 x5 x6 x7 x8 x9]; %cada columna es un piloto
X=double(X);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% PCA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[V,D,Media] = pc_evectors(X,Nv);


for i = 1:N
    Xm(:,i) = X(:,i) - Media; %Quito la media los datos para proyectar y luego reconstruir
   
end;

p=V'*Xm; %proyeccion de los pilotos en el subespacio
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%% RECONSTRUCCION %%%%%%%%%%%%%%%%%%%%%%%%%

num=0;
im_reco=[];
for num_vec=[1 2 3 4 5 6 7 8]  %numero de vectores que utilizo en la recosntruccion

	num=num+D(num_vec);
	y=V(:,1:num_vec)*p(1:num_vec,:); %reconstruccion
	ym=y+Media*ones(1,N);
	energia(num_vec)=num/sum(D);  %energia en cada recosntruccion (maximo=1)
	im_reco=[im_reco ym(:,numero)]; %imagenes de los pilotos reconstruidas
    
end

e=Inf*ones(1,Nv);
for i=1:Nv
    e(i)=norm(X(:,numero)-im_reco(:,i)); % error en la reconstruccion
end

%%%%%%%%%%%%%%%%%%%%%% Visualizacin %%%%%%%%%%%%%%%%%%%%%%%%%%%

if display==1
figure
ver_matriz(X',Nx,Ny)
title('alumno Xxxx Yyyy Zzzz')
figure
ver_matriz(im_reco',Nx,Ny)
title('alumno Xxxx Yyyy Zzzz')
figure
bar(e)
title('alumno Xxxx Yyyy Zzzz')
end







%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ver_matriz(matriz,x,y)
    [rows,cols]=size(matriz);
%    vistas=ceil(sqrt(rows));
    
    for i=1:rows
        subplot(1,rows,i)
        imagen=uint8(matriz(i,:));
        ver(imagen,x,y)
    end


function ver(imagen,x,y)
%imshow(uint8(reshape(objeto(:,vista),128,128)))
%colormap(gray(256));
imshow((reshape(imagen,x,y,3)));


